#ifndef CONSOLE_H__
#define CONSOLE_H__

/********************************************************************
	created:	2009/10/09
	filename: 	Console.h
	author:		Daniel Wellman
	
	purpose:	Provides clean and convenient access to a double-buffered
				Win32 console

	License: This software may be reproduced and distributed free of charge as
	as long as credit is given to the author. The author assumes no
	responsibility. The software is provided as is.
*********************************************************************/
#include <utility>
#include "NonCopyable.h"

namespace LuaAutomata
{
	enum Color;
	class ConsoleBuffer;

	class Console : public NonCopyable
	{
	public:
		Console(unsigned width, unsigned height);
		~Console();
		void WriteChar(char charToWrite, unsigned x, unsigned y, Color foreground, Color background);
		const std::pair<unsigned, unsigned> GetCursorPosition() const;
		bool IsLeftMouseDown() const;
		bool IsRightMouseDown() const;
		bool IsEnterClicked() const;
		bool IsEscapeClicked() const;
		bool IsLeftMouseClicked() const;
		void Clear();
		void Update();
		void Draw();
	private:

		const unsigned m_height;
		const unsigned m_width;

		bool m_isLeftMouseDown, m_wasLeftMouseDown;
		bool m_isRightMouseDown;
		bool m_isEnterDown, m_wasEnterDown;
		bool m_isEscapeDown, m_wasEscapeDown;

		int m_mouseX, m_mouseY;

		ConsoleBuffer* m_frontBuffer;
		ConsoleBuffer* m_backBuffer;
	};
}
#endif // CONSOLE_H__
