#include "ConsoleBuffer.h"
#include <assert.h>
#include "Color.h"

namespace LuaAutomata
{
	ConsoleBuffer::ConsoleBuffer(unsigned width, unsigned height)
		: m_charBuffer(0), 
		m_screenBuffer(0),
		m_width(width), 
		m_height(height)
	{
		//Create an unshared full access screen buffer
		m_screenBuffer = CreateConsoleScreenBuffer(GENERIC_WRITE,
			0,
			0,
			CONSOLE_TEXTMODE_BUFFER,
			0);

		assert(m_screenBuffer);

		m_charBuffer = new CHAR_INFO[m_width * m_height];
	}

	ConsoleBuffer::~ConsoleBuffer()
	{
		//Clean up on aisle 3
		delete [] m_charBuffer;
		if (m_screenBuffer)
			CloseHandle(m_screenBuffer);
	}

	void ConsoleBuffer::SetActive()
	{
		const short width = static_cast<short>(m_width);
		const short height = static_cast<short>(m_height);

		const COORD bufferSize = {width, height};
		const COORD bufferLoc = {0, 0};
		SMALL_RECT srctWriteRect = {0, 0, width - 1, height - 1};

		//Write the char buffer into the screen buffer
		WriteConsoleOutput( 
			m_screenBuffer,
			m_charBuffer,
			bufferSize,
			bufferLoc,
			&srctWriteRect);

		//Set the buffer active
		SetConsoleActiveScreenBuffer(m_screenBuffer);
	}

	namespace
	{
		unsigned GetIndex(unsigned x, unsigned y, unsigned width)
		{
			return x + y * width;
		}
	}

	void ConsoleBuffer::WriteChar(char charToWrite, unsigned x, unsigned y, Color foreground, Color background)
	{
		assert(x < m_width);
		assert(y < m_height);

		const unsigned index = GetIndex(x, y, m_width);
		m_charBuffer[index].Char.UnicodeChar = charToWrite;
		m_charBuffer[index].Attributes = static_cast<unsigned short>(foreground + background * 16);
	}

	void ConsoleBuffer::Clear()
	{
		const unsigned size = m_width * m_height;

		for (unsigned i = 0; i < size; ++i)
		{
			m_charBuffer[i].Char.UnicodeChar = ' ';
			m_charBuffer[i].Attributes = 0;
		}
	}
}