#include "FrameLimiter.h"

namespace LuaAutomata
{
	FrameLimiter::FrameLimiter(double fps)
		: m_fps(fps),
		m_frequency(0), 
		m_timeBetweenFrames(0),
		m_secondsPerFrequency(0.0f),
		m_nextTime(0), 
		m_currentTime(0), 
		m_lastTime(0)
	{
		LARGE_INTEGER tempInt;

		::QueryPerformanceFrequency(&tempInt);
		m_frequency = tempInt.QuadPart;

		SetMaxFps(m_fps);

		m_secondsPerFrequency = 1.0 / static_cast<double>(m_frequency);


		::QueryPerformanceCounter(&tempInt);
		m_lastTime = m_nextTime = tempInt.QuadPart;
	}

	double FrameLimiter::Tick()
	{
		LARGE_INTEGER currentTime;
		::QueryPerformanceCounter(&currentTime);
		m_currentTime = currentTime.QuadPart;

		const double delta = static_cast<double>((m_currentTime - m_lastTime) * m_secondsPerFrequency);
		m_lastTime = m_currentTime;
		return delta;
	}

	void FrameLimiter::ResetTime()
	{
		m_nextTime = m_currentTime + m_timeBetweenFrames;
	}

	bool FrameLimiter::IsTimeUp() const
	{
		return m_currentTime > m_nextTime;
	}

	void FrameLimiter::SetMaxFps(double fps)
	{
		m_fps = fps;
		m_timeBetweenFrames = static_cast<LONGLONG>(m_frequency / m_fps);
	}
}