#include "LuaGame.h"
#include "lua.hpp"
#include <assert.h>
#include "LuaAutomataLib.h"
#include <Windows.h>

namespace LuaAutomata
{
	int AtPanic(lua_State* const L)
	{
		const char* const reason = lua_tostring(L, -1);
		::MessageBoxA(NULL, reason, "Lua panic!", MB_ICONERROR | MB_OK);
		return 0;
	}

	LuaGame::LuaGame(Console& console, unsigned width, unsigned height) 
		: m_exiting(false),
		L(luaL_newstate())
	{
		lua_atpanic(L, AtPanic);

		//Open main libs
		luaL_openlibs(L);

		//Set the size of our grid into Lua globals
		lua_pushinteger(L, width);
		lua_setglobal(L, "WIDTH");

		lua_pushinteger(L, height);
		lua_setglobal(L, "HEIGHT");

		LuaAutomataLib::AddMethods(L, console);

		//Load all of the files in our list
		if (luaL_loadfile(L, "FilesToLoad.lua") != 0 || lua_pcall(L, 0, 0, 0) != 0)
			Error(lua_tostring(L, -1));
	}

	LuaGame::~LuaGame()
	{
		lua_close(L);
	}

	void LuaGame::Update(float dt)
	{
		//Push the name of the Update function onto the stack
		lua_getglobal(L, "Update");
		lua_pushnumber(L, dt);
		//Run the update function
		if (lua_pcall(L, 1, 1, 0) != 0)
			Error(lua_tostring(L, -1));
		else if (lua_isboolean(L,-1) && lua_toboolean(L, -1) == 1) //Update returns true when it wants to exit
			m_exiting = true;
	}

	void LuaGame::Draw()
	{
		lua_getglobal(L, "Draw");
		if (lua_pcall(L, 0, 0, 0) != 0)
			Error(lua_tostring(L, -1));
	}

	void LuaGame::Error(const char* const error)
	{
		::MessageBoxA(NULL, error, "Lua error!", MB_ICONWARNING | MB_OK);
		m_exiting = true;
	}
}