#include "LuaGame.h"
#include "Console.h"
#include "FrameLimiter.h"

int main()
{
	_CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);

	const unsigned WIDTH = 25;
	const unsigned HEIGHT = 80;

	LuaAutomata::Console console(HEIGHT, WIDTH);
	LuaAutomata::LuaGame game(console, HEIGHT, WIDTH);

	LuaAutomata::FrameLimiter limiter(30.0);

	float accumulator = 0.0f;
	const float TIME_STEP = .02f;
	//Start game loop
	while (!game.IsExiting())
	{
		const float delta = static_cast<float>(limiter.Tick());
		accumulator += delta;
		if (accumulator >= TIME_STEP)
		{
			console.Update();
			game.Update(TIME_STEP);
			//Not the most accurate simulation, but keeps it from falling behind on slower systems
			accumulator = 0.0f;
		}
		if (limiter.IsTimeUp())
		{
			game.Draw();
			console.Draw();

			limiter.ResetTime();
		}
	}
	return 0;
}